package ru.dbotthepony.kstarbound.defs.liquid

import com.google.gson.GsonBuilder
import ru.dbotthepony.kstarbound.io.KConcreteTypeAdapter
import ru.dbotthepony.kstarbound.registerTypeAdapter
import ru.dbotthepony.kvector.vector.Color

data class LiquidDefinition(
	val name: String,
	val liquidId: Int,
	val description: String = "...",
	val tickDelta: Int = 1,
	val color: Color,
	val itemDrop: String? = null,
	val statusEffects: List<String> = listOf(),
	val interactions: List<Interaction> = listOf(),
	val texture: String,
	val bottomLightMix: Color,
	val textureMovementFactor: Double,
) {
	data class Interaction(val liquid: Int, val liquidResult: Int? = null, val materialResult: String? = null) {
		init {
			require(liquidResult != null || materialResult != null) { "Both liquidResult and materialResult are missing" }
		}
	}

	companion object {
		val ADAPTER = KConcreteTypeAdapter.Builder(LiquidDefinition::class)
			.plain(LiquidDefinition::name)
			.plain(LiquidDefinition::liquidId)
			.plain(LiquidDefinition::description)
			.plain(LiquidDefinition::tickDelta)
			.plain(LiquidDefinition::color)
			.plain(LiquidDefinition::itemDrop)
			.list(LiquidDefinition::statusEffects)
			.list(LiquidDefinition::interactions)
			.plain(LiquidDefinition::texture)
			.plain(LiquidDefinition::bottomLightMix)
			.plain(LiquidDefinition::textureMovementFactor)
			.build()

		val INTERACTION_ADAPTER = KConcreteTypeAdapter.Builder(Interaction::class)
			.plain(Interaction::liquid)
			.plain(Interaction::liquidResult)
			.plain(Interaction::materialResult)
			.build()

		fun registerGson(gsonBuilder: GsonBuilder) {
			gsonBuilder.registerTypeAdapter(ADAPTER)
			gsonBuilder.registerTypeAdapter(INTERACTION_ADAPTER)
		}
	}
}
